# FILE EXTENSION
DOCX_FILE_EXTENSION = ".docx"
JSON_FILE_EXTENSION = ".json"
PDF_FILE_EXTENSION = ".pdf"
TXT_FILE_EXTENSION = ".txt"

# ENCODING
UTF_8 = "utf-8"

# PARSING
PUNCTUATION = "—"  # holds punctuation characters that string.punctuation does not include.



import json
from typing import Optional

from wordfreak.parser.DocxParser import DocxParser
from wordfreak.parser.PdfParser import PdfParser
from wordfreak.parser.TxtParser import TxtParser
from wordfreak.util.constants import JSON_FILE_EXTENSION, TXT_FILE_EXTENSION, PDF_FILE_EXTENSION, DOCX_FILE_EXTENSION


def extractWordFrequencies(inputFilePath: str, outputFilePath: Optional[str] = None) -> dict[str, int]:
    """
    Takes a file path and extracts all word frequencies from it as a Python dictionary.

    If outputFilePath is given, it will also save the results to that file as JSON.

    inputFilePath: Path to file to extract work frequencies from.
    outputFilePath: Path to file to save word frequencies to (must be .json file).
    """

    if outputFilePath is not None and not outputFilePath.lower().endswith(JSON_FILE_EXTENSION):
        raise ValueError(f"Output file must be a .json file.")

    if inputFilePath.lower().endswith(TXT_FILE_EXTENSION):
        wordFrequencies = TxtParser.getWordFrequency(inputFilePath)
    elif inputFilePath.lower().endswith(PDF_FILE_EXTENSION):
        wordFrequencies = PdfParser.getWordFrequency(inputFilePath)
    elif inputFilePath.lower().endswith(DOCX_FILE_EXTENSION):
        wordFrequencies = DocxParser.getWordFrequency(inputFilePath)
    else:
        raise ValueError(f"Filetype not supported for parsing (tried to parse file at '{inputFilePath}').")

    # sort word frequencies by number of occurrences
    orderedWordFreq = dict(sorted(wordFrequencies.items(), reverse=True, key=lambda item: item[1]))

    # save to JSON file if requested
    if outputFilePath is not None:
        with open(outputFilePath, "w+") as file:
            json.dump(orderedWordFreq, file)

    return orderedWordFreq


def pythonizeWordFrequencies(jsonFilePath: str) -> dict[str, int]:
    """
    Takes a file path to a JSON file that holds word frequencies and returns it as a Python dictionary.
    After word frequencies have been extracted by extractWordFrequencies(), the resulting JSON file can be fed into this method to Pythonize.

    jsonFilePath: Path to a file to Pythonize (must be .json file).
    """
    with open(jsonFilePath) as json_file:
        wordFrequencies: dict = json.load(json_file)

    # make sure this is a valid word frequencies file/dict
    if not all(isinstance(count, int) for count in wordFrequencies.values()):
        raise ValueError("Word Frequencies not formatted correctly, values must by type 'int'.")

    return wordFrequencies



from wordfreak.parser.Parser import Parser
from wordfreak.util.constants import UTF_8


class TxtParser(Parser):
    """
    For use parsing .txt files.
    """

    @classmethod
    def getWordFrequency(cls, pathToTxtFile: str) -> dict[str, int]:
        # get all word frequencies from the given .txt file
        with open(pathToTxtFile, encoding=UTF_8) as file:
            lines = file.readlines()
        return cls._getWordFrequencyFromLines(lines)



import docx2txt

from wordfreak.parser.Parser import Parser


class DocxParser(Parser):
    """
    For use parsing .docx files.
    """

    @classmethod
    def getWordFrequency(cls, pathToPdfFile: str) -> dict[str, int]:
        # get all word frequencies from the given .docx file
        lines = docx2txt.process(pathToPdfFile).split()
        return cls._getWordFrequencyFromLines(lines)



from PyPDF2 import PdfReader

from wordfreak.parser.Parser import Parser


class PdfParser(Parser):
    """
    For use parsing .pdf files.
    """

    @classmethod
    def getWordFrequency(cls, pathToPdfFile: str) -> dict[str, int]:
        # get all word frequencies from the given .pdf file
        lines = list()
        reader = PdfReader(pathToPdfFile)
        for page in reader.pages:
            lines.append(page.extract_text())
        return cls._getWordFrequencyFromLines(lines)



import setuptools

pkg_vars = dict()
with open("wordfreak/_version.py") as f:
    exec(f.read(), pkg_vars)

with open("README.md") as f:
    readMe = f.read()

setuptools.setup(
    name="wordfreak",
    version=pkg_vars["__version__"],
    author="Joey Greco",
    author_email="joeyagreco@gmail.com",
    description="Word Freak is a Python library that extracts word frequencies from files.",
    long_description_content_type="text/markdown",
    long_description=readMe,
    license="MIT",
    packages=setuptools.find_packages(exclude=("test", "docs")),
    install_requires=["PyPDF2",
                      "docx2txt",
                      "setuptools"]
)
